"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SSHClient = void 0;
/* eslint-disable no-underscore-dangle */
const ssh2_1 = require("ssh2");
const SSHForward_1 = require("./SSHForward");
const LOCAL_FORWARD_PORT = 0;
class SSHClient extends ssh2_1.Client {
    constructor() {
        super();
        this._tunnels = new Set();
        this._childs = new Set();
        this._justHop = false;
        this.on('close', this._internalOnClose);
    }
    // TODO: allow connect through socket
    connect(config) {
        const { tunnels } = config;
        if (tunnels) {
            this.on('ready', () => {
                tunnels.forEach(({ localPort, remoteHost, remotePort }) => {
                    this.forwardPort(localPort, remoteHost, remotePort);
                });
            });
        }
        if (!config.through) {
            super.connect(config);
            return;
        }
        if (config.through instanceof SSHClient) {
            this._connectThrough(config.through, config);
        }
        else {
            const parent = new SSHClient();
            parent._justHop = true;
            parent.connect(config.through);
            parent.on('ready', () => {
                this._connectThrough(parent, config);
            });
        }
    }
    releaseParent() {
        if (!this._parent) {
            return false;
        }
        this._parent._justHop = false;
        return this._parent;
    }
    forwardPort(localPort, remoteAdress, remotePort) {
        return new SSHForward_1.SSHForward(this, this._tunnels, {
            localPort,
            remoteAdress,
            remotePort
        });
    }
    forwardSocket(dstIP, dstPort, onConnection) {
        this.forwardOut('', 0, dstIP, dstPort, onConnection);
    }
    getPortForwards() {
        return [...this._tunnels];
    }
    _connectThrough(parent, config) {
        parent.forwardOut('127.0.0.1', LOCAL_FORWARD_PORT, config.host, config.port, (error, channel) => {
            if (error) {
                throw error;
            }
            config.sock = channel;
            this._parent = parent;
            super.connect(config);
            this.on('ready', () => {
                parent._childs.add(this);
            });
        });
    }
    _internalOnClose() {
        // close childs
        this._childs.forEach(child => {
            console.log('closing childs');
            child.end();
        });
        if (this._parent) {
            // remove from parent
            this._parent._childs.delete(this);
            // close parent if this was the last child and the parent is just a hop parent
            if (!this._parent._childs.size && this._parent._justHop) {
                this._parent.end();
            }
        }
    }
}
exports.SSHClient = SSHClient;
