"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SSHForward = void 0;
const net_1 = require("net");
class SSHForward {
    constructor(owner, siblings, { localPort, remoteAdress, remotePort }) {
        this._connections = new Set();
        this._siblings = siblings;
        this._owner = owner;
        this._server = net_1.createServer(serverStream => {
            this._owner.forwardSocket(remoteAdress, remotePort, (err, sshStream) => {
                if (err) {
                    throw err;
                }
                serverStream.pipe(sshStream).pipe(serverStream);
                this._connections.add(serverStream);
                sshStream.on('close', () => {
                    this._connections.delete(serverStream);
                });
            });
        });
        this._server.listen(localPort);
        this._siblings.add(this);
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this._server.close(error => {
                    if (error) {
                        reject(error);
                    }
                    else {
                        resolve();
                    }
                });
                // end open connections
                this._connections.forEach(connection => connection.end());
                this._siblings.delete(this);
            });
        });
    }
}
exports.SSHForward = SSHForward;
